{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2022 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{% extends "invenio_communities/details/members/base.html" %}

{%- block javascript %}
  {{ super() }}
  {{ webpack['invenio-communities-invitations.js'] }}
{%- endblock javascript %}

{% set active_menu_item = 'invitations' %}
{% set active_menu_tab = 'members' %}

{%- block settings_body %}

  <div class="thirteen wide column">
    <div
      id="community-invitations-search-root"
      data-communities-roles='{{ config.COMMUNITIES_ROLES | tojson }}'
      data-permissions='{{ permissions | tojson }}'
      data-community='{{ community | tojson }}'
      data-community-allow-group-invites='{{ config.COMMUNITIES_ALLOW_GROUP_INVITES | tojson }}'
      data-invenio-search-config='{{
        search_app_communities_invitations_config(app_id="communities-invitations-search",
        endpoint=community["links"]["invitations"]) | tojson }}'>
    </div>
  </div>


{%- endblock settings_body %}
